package	jp.co.sds_corp.hibiya.connector;

import java.util.Vector;

import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* {@link IChecker}の情報を取得するためのインターフェースです。
*/
public abstract class AbstractConnector extends AbstractService
	implements IConnector
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	transient Vector<ICheckerListener> vecStat = new Vector<ICheckerListener>();
	transient Vector<IConnectionListener> vecLsn = new Vector<IConnectionListener>();
	boolean	displayOnBlank = true;	// 空でも表示するフラグ
	boolean	status = false;
	int	checkInterval = 60000;	// チェック間隔
	transient Thread	chkThr;

	AbstractConnector() {
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		String var;
		var = config.getAttributes().get( "displayOnBlank", "true" );
		if ( !var.equals( "true" )) {
			displayOnBlank = false;
		}
		var = config.getAttributes().get( "checkInterval", "60000" );
		checkInterval = Integer.valueOf( var ).intValue();
	}

	public void start() {
		if ( chkThr == null ) {
			chkThr = new Thread( new ConnectionChecker() );
			chkThr.setName( "ConnectionChecker" );
			chkThr.start();
		}
		logger.debug( this, "started.[" + getName() + "]" );
	}

	public void stop() {
		logger.debug( this, "stop...[" + getName() + "]" );
		if ( chkThr != null ) {
			chkThr.interrupt();
			try {
				chkThr.join();
			} catch ( Exception ex ) {
			}
			chkThr = null;
		}
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		if ( !isDisplayOnBlank()) {
			config.setAttribute( "displayOnBlank", "false" );
		}
		config.setAttribute( "checkInterval", String.valueOf( checkInterval ));
	}

	/**
	* 接続状態を検査します。
	*/
	public abstract boolean checkConnection() throws Exception;

	public class ConnectionChecker implements Runnable {
		public void run() {
			for ( ;; ) {
				if ( Thread.currentThread().interrupted()) {
					break;
				}
				boolean bRet = false;
				try {
					bRet = checkConnection();
				} catch ( InterruptedException ex ) {
					break;
				} catch ( Exception ex ) {
				}
				if ( isConnected()) {
					if ( !bRet ) {
						logger.info( this, "disconnected.[" + getName() + "]" );
						fireDisconnected( new ConnectionEvent( this ));
					}
				} else {
					if ( bRet ) {
						logger.info( this, "connected.[" + getName() + "]" );
						fireConnected( new ConnectionEvent( this ));
					}
				}
				Thread.yield();
				try {
					Thread.sleep( getCheckInterval() );
				} catch ( Exception ex ) {
					break;
				}
			}
		}
	}

	/**
	* 状態を取得します。
	* @return	状態
	*/
	public synchronized boolean isConnected() {
		return this.status;
	}
	/**
	* 状態を設定します。
	* @param	var		状態
	*/
	protected synchronized void setConnected( boolean var ) {
		this.status = var;
	}

	/**
	* チェック間隔を設定します。
	* @param	var		設定するデータ
	*/
	public void setCheckInterval( int var ) {
		this.checkInterval = var;
	}
	/**
	* チェック間隔を取得します。
	* @return	チェック間隔
	*/
	public int getCheckInterval() {
		return checkInterval;
	}

	/**
	* 接続状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addConnectionListener( IConnectionListener l ) {
		vecLsn.add( l );
	}

	/**
	* 接続状態の変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeConnectionListener( IConnectionListener l ) {
		vecLsn.remove( l );
	}

	/**
	* 接続状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireConnected( ConnectionEvent ev ) {
		setConnected( true );
		ConnectionEvent nev = new ConnectionEvent( this );
		IConnectionListener l;
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			l = (IConnectionListener )vecLsn.get( i );
			try {
				l.connected( nev );
			} catch ( Exception ex ) {
				logger.error( this, "connect notify error"
							+ "[" + getName() + "]->[" + l + "]" + ex );
			}
		}
	}

	/**
	* 接続状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireDisconnected( ConnectionEvent ev ) {
		setConnected( false );
		ConnectionEvent nev = new ConnectionEvent( this );
		IConnectionListener l;
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			l = (IConnectionListener )vecLsn.get( i );
			try {
				l.disconnected( nev );
			} catch ( Exception ex ) {
				logger.error( this, "disconnect notify error"
							+ "[" + getName() + "]->[" + l + "]" + ex );
			}
		}
	}

	/**
	* 空でも表示するフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDisplayOnBlank( boolean var ) {
		this.displayOnBlank = var;
	}
	/**
	* 空でも表示するフラグを取得します。
	* @return	空でも表示するフラグ
	*/
	public boolean isDisplayOnBlank() {
		return displayOnBlank;
	}

	/**
	* 状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addCheckerListener( ICheckerListener l ) {
		vecStat.add( l );
	}

	/**
	* レート変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeCheckerListener( ICheckerListener l ) {
		vecStat.remove( l );
	}

	/**
	* 状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireCheckerChange( CheckerChangeEvent ev ) throws Exception {
		for ( int i = 0; i < vecStat.size(); i++ ) {
			ICheckerListener l = (ICheckerListener )vecStat.get( i );
			ev.setConnector( this );
			l.checkerChanged( ev );
		}
	}

}
